﻿using Microsoft.Extensions.DependencyInjection;
using SlnMesnac.Config;
using SlnMesnac.Repository.service;
using SlnMesnac.Repository.service.Impl;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Text;

namespace SlnMesnac.Repository
{
    public static class SqlsugarSetup
    {
        /// <summary>
        /// 注册SqlSugar
        /// </summary>
        /// <param name="services"></param>
        public static void AddSqlSugarSetup(this IServiceCollection services)
        {
            services.AddSingleton<ISqlSugarClient>(x =>
            {
                var appConfig = x.GetService<AppConfig>();
                SqlSugarScope Db = new SqlSugarScope(new List<ConnectionConfig>()
                {
                    new ConnectionConfig()
                    {
                        ConfigId = "mes",
                        DbType = DbType.SqlServer,
                        ConnectionString = appConfig.mesConnStr,
                        InitKeyType = InitKeyType.Attribute,
                        IsAutoCloseConnection = true,
                    },
                    new ConnectionConfig()
                    {
                        ConfigId = "mcs",
                        DbType = DbType.Oracle,
                        ConnectionString = appConfig.mcsConnStr,
                        InitKeyType = InitKeyType.Attribute,
                        IsAutoCloseConnection = true,
                    }
                },
                db =>
                {
                    db.Aop.OnLogExecuting = (sql, pars) => { };
                });

                return Db;
            });
        }

        /// <summary>
        /// 注册Repository、Service服务
        /// </summary>
        /// <param name="services"></param>
        public static void AddServices(this IServiceCollection services)
        {
            //services.AddSingleton<Repository<BaseUser>>();
            services.AddSingleton(typeof(Repository<>));
            services.AddSingleton<IBaseUserService, BaseUserServiceImpl>();
            services.AddSingleton<IBaseMaterialService, BaseMaterialServiceImpl>();
        }
    }
}
