﻿using Microsoft.Extensions.Logging;
using SlnMesnac.Model.domain;
using SlnMesnac.Common;
using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Text;

namespace SlnMesnac.Repository.service.Impl
{
    public class BaseMaterialServiceImpl : IBaseMaterialService
    {
        private Repository<BaseMaterialInfo> _repository;

        private ILogger<BaseMaterialServiceImpl> _logger;

        public BaseMaterialServiceImpl(Repository<BaseMaterialInfo> repository, ILogger<BaseMaterialServiceImpl> logger)
        {
            _repository = repository;
            _logger = logger;
        }

        /// <summary>
        /// 通过物料编码获取物料信息
        /// </summary>
        /// <param name="materialCode"></param>
        /// <returns></returns>
        public BaseMaterialInfo GetMaterialInfoByMaterialCode(string materialCode)
        {
            BaseMaterialInfo materialInfo = null;
            try
            {
                materialInfo = _repository.GetFirst(x => x.MaterialCode == materialCode);
            }
            catch (Exception ex)
            {
                _logger.LogError($"根据物料编号获取物料信息异常:{ex.Message}");
            }
            return materialInfo;
        }

        /// <summary>
        /// 通过SAP物料编码获取物料信息
        /// </summary>
        /// <param name="sapMaterialCode"></param>
        /// <returns></returns>
        public BaseMaterialInfo GetMaterialInfoBySapMaterialCode(string sapMaterialCode)
        {
            BaseMaterialInfo materialInfo = null;
            try
            {
                materialInfo = _repository.GetFirst(x => x.SAPMaterialCode == sapMaterialCode);
            }
            catch (Exception ex)
            {
                _logger.LogError($"根据SAP物料编号获取物料信息异常:{ex.Message}");
            }
            return materialInfo;
        }

        /// <summary>
        /// 获取所有的物料信息
        /// </summary>
        /// <returns></returns>
        public List<BaseMaterialInfo> GetMaterialInfos()
        {
            List<BaseMaterialInfo> materialInfos = null;
            try
            {
                materialInfos = _repository.GetList();
            }
            catch (Exception ex)
            {
                _logger.LogError($"获取物料信息异常:{ex.Message}");
            }
            return materialInfos;
        }

        /// <summary>
        /// 通过物料类别获取物料信息
        /// </summary>
        /// <param name="majorTypeId">物料大类</param>
        /// <param name="minorTypeId">物料细类</param>
        /// <returns></returns>
        public List<BaseMaterialInfo> GetMaterialInfosByMaterialType(int majorTypeId, string minorTypeId)
        {
            List<BaseMaterialInfo> materialInfos = null;
            try
            {
                Expression<Func<BaseMaterialInfo, bool>> exp = x => true;

                if (majorTypeId != 0)
                {
                    exp = exp.And(x => x.MajorTypeID == majorTypeId);
                }
                else if (!string.IsNullOrEmpty(minorTypeId))
                {
                    exp = exp.And(x => x.MinorTypeID == minorTypeId);
                }

                materialInfos = _repository.GetList(exp);
            }
            catch (Exception ex)
            {
                _logger.LogError($"通过物料类型获取物料信息异常:{ex.Message}");
            }
            return materialInfos;
        }
    }
}

